@@ -46,7 +46,7 @@ bool Model::FnEntry::checkPoly(const EnvI& env, const Type& t) {
   }
   if (t.structBT()) {
     StructType* st = env.getStructType(t);
-    for (size_t i = 0; i < st->size(); ++i) {
+    for (unsigned int i = 0; i < st->size(); ++i) {
       if (checkPoly(env, (*st)[i])) {
         return true;
       }
@@ -298,7 +298,7 @@ struct TIIDInfo {
 void TypeInst::collectTypeIds(std::unordered_map<ASTString, size_t>& seen_tiids,
                               std::vector<TIIDInfo>& type_ids) const {
   auto* al = Expression::cast<ArrayLit>(domain());
-  for (size_t i = 0; i < al->size(); i++) {
+  for (unsigned int i = 0; i < al->size(); i++) {
     auto* ti = Expression::cast<TypeInst>((*al)[i]);
     if (ti->type().bt() == Type::BT_TOP) {
       // If type is top, either this is a TIId, or it is caused by <>
@@ -398,7 +398,7 @@ void Model::addPolymorphicInstances(EnvI& env, Model::FnEntry& fe, std::vector<F
         // New complete instance
         // First, update cur types
         auto* tis = Expression::cast<ArrayLit>(paramtuple->domain());
-        for (size_t i = 0; i < tis->size(); ++i) {
+        for (unsigned int i = 0; i < tis->size(); ++i) {
           cur.t[i] = Expression::type((*tis)[i]);
           if (cur.t[i].bt() == Type::BT_TUPLE && cur.t[i].typeId() == 0) {
             env.registerTupleType(Expression::cast<TypeInst>((*tis)[i]));
@@ -724,7 +724,7 @@ void Model::checkFnValid(EnvI& env, std::vector<TypeError>& errors) {
       }
       if (fi->ann().contains(env.constants.ann.output_only)) {
         std::vector<Type> tys(fi->paramCount());
-        for (int i = 0; i < fi->paramCount(); ++i) {
+        for (unsigned int i = 0; i < fi->paramCount(); ++i) {
           tys[i] = fi->param(i)->type();
           tys[i].mkPar(env);
         }
@@ -749,7 +749,7 @@ void Model::checkFnValid(EnvI& env, std::vector<TypeError>& errors) {
             "FlatZinc builtin functions must be predicates (i.e., have `var bool` return type)");
         continue;
       }
-      for (int i = 0; i < fi->paramCount(); ++i) {
+      for (unsigned int i = 0; i < fi->paramCount(); ++i) {
         const Type& t = fi->param(i)->type();
         if (t.isOpt() || t.structBT() || t.bt() == Type::BT_TOP) {
           errors.emplace_back(
@@ -873,7 +873,7 @@ FunctionI* Model::matchReification(EnvI& env, const ASTString& id, const std::ve
       // is at least as specific as the reification. That is the parameters of the matching
       // half-reification declaration should be the same or subtypes of the matching reification.
       assert(imp_decl->paramCount() == reif_decl->paramCount());
-      for (size_t i = 0; i < imp_decl->paramCount(); ++i) {
+      for (unsigned int i = 0; i < imp_decl->paramCount(); ++i) {
         Type a = imp_decl->param(i)->ti()->type();
         Type b = reif_decl->param(i)->ti()->type();
         if (!env.isSubtype(a, b, strictEnums)) {
@@ -1065,7 +1065,8 @@ int first_overloaded(EnvI& env, const std::vector<Model::FnEntry>& v_f, int i_f)
 
 bool Model::sameOverloading(EnvI& env, const std::vector<Expression*>& args, FunctionI* f,
                             FunctionI* g) const {
-  if (f->id() == env.constants.varRedef->id() || g->id() == env.constants.varRedef->id()) {
+  if (f->isMonomorphised() || g->isMonomorphised() || f->id() == env.constants.varRedef->id() ||
+      g->id() == env.constants.varRedef->id()) {
     return false;
   }
   const Model* m = this;
@@ -1097,7 +1098,7 @@ bool Model::sameOverloading(EnvI& env, const std::vector<Expression*>& args, Fun
     // not the same number of overloaded versions
     return false;
   }
-  for (; first_i_f <= i_f; first_i_f++, first_i_g++) {
+  for (; first_i_f <= static_cast<unsigned int>(i_f); first_i_f++, first_i_g++) {
     if (!(v_f[first_i_f].t == v_g[first_i_g].t)) {
       // one of the overloaded versions does not agree in the types
       return false;
